@@ -5,9 +5,11 @@ from typing import Any, List
 from pathlib import Path
 
 from erdpy import cli_shared, errors, projects, utils
-from erdpy.accounts import Account, Address
+from erdpy.accounts import Account, Address, LedgerAccount
+from erdpy.cli_output import CLIOutputBuilder
 from erdpy.contracts import CodeMetadata, SmartContract
 from erdpy.projects import load_project
+from erdpy.projects.core import get_project_paths_recursively
 from erdpy.proxy.core import ElrondProxy
 from erdpy.transactions import Transaction
 
@@ -15,38 +17,62 @@ logger = logging.getLogger("cli.contracts")
 
 
 def setup_parser(args: List[str], subparsers: Any) -> Any:
-    parser = cli_shared.add_group_subparser(subparsers, "contract", "Build, deploy and interact with Smart Contracts")
+    parser = cli_shared.add_group_subparser(subparsers, "contract", "Build, deploy, upgrade and interact with Smart Contracts")
     subparsers = parser.add_subparsers()
 
-    sub = cli_shared.add_command_subparser(subparsers, "contract", "new", "Create a new Smart Contract project based on a template.")
+    sub = cli_shared.add_command_subparser(subparsers, "contract", "new",
+                                           "Create a new Smart Contract project based on a template.")
     sub.add_argument("name")
     sub.add_argument("--template", required=True, help="the template to use")
-    sub.add_argument("--directory", type=str, default=os.getcwd(), help="🗀 the parent directory of the project (default: current directory)")
+    sub.add_argument("--directory", type=str, default=os.getcwd(),
+                     help="🗀 the parent directory of the project (default: current directory)")
     sub.set_defaults(func=create)
 
-    sub = cli_shared.add_command_subparser(subparsers, "contract", "templates", "List the available Smart Contract templates.")
+    sub = cli_shared.add_command_subparser(subparsers, "contract", "templates",
+                                           "List the available Smart Contract templates.")
     sub.set_defaults(func=list_templates)
 
-    sub = cli_shared.add_command_subparser(subparsers, "contract", "build", "Build a Smart Contract project using the appropriate buildchain.")
+    sub = cli_shared.add_command_subparser(subparsers, "contract", "build",
+                                           "Build a Smart Contract project using the appropriate buildchain.")
     _add_project_arg(sub)
+    _add_recursive_arg(sub)
     sub.add_argument("--debug", action="store_true", default=False, help="set debug flag (default: %(default)s)")
-    sub.add_argument("--no-optimization", action="store_true", default=False, help="bypass optimizations (for clang) (default: %(default)s)")
+    sub.add_argument("--no-optimization", action="store_true", default=False,
+                     help="bypass optimizations (for clang) (default: %(default)s)")
+    sub.add_argument("--no-wasm-opt", action="store_true", default=False,
+                     help="do not optimize wasm files after the build (default: %(default)s)")
     sub.add_argument("--cargo-target-dir", type=str, help="for rust projects, forward the parameter to Cargo")
-    sub.add_argument("--wasm-symbols", action="store_true", default=False, help="for rust projects, does not strip the symbols from the wasm output. Useful for analysing the bytecode. Creates larger wasm files. Avoid in production (default: %(default)s)")
-    sub.add_argument("--wasm-name", type=str, help="for rust projects, optionally specify the name of the wasm bytecode output file")
+    sub.add_argument("--wasm-symbols", action="store_true", default=False,
+                     help="for rust projects, does not strip the symbols from the wasm output. Useful for analysing the bytecode. Creates larger wasm files. Avoid in production (default: %(default)s)")
+    sub.add_argument("--wasm-name", type=str,
+                     help="for rust projects, optionally specify the name of the wasm bytecode output file")
+    sub.add_argument("--skip-eei-checks", action="store_true", default=False, help="skip EEI compatibility checks (default: %(default)s)")
+    sub.add_argument("--ignore-eei-checks", action="store_true", default=False, help="ignore EEI compatibility errors (default: %(default)s)")
     sub.set_defaults(func=build)
 
     sub = cli_shared.add_command_subparser(subparsers, "contract", "clean", "Clean a Smart Contract project.")
     _add_project_arg(sub)
+    _add_recursive_arg(sub)
     sub.set_defaults(func=clean)
 
     sub = cli_shared.add_command_subparser(subparsers, "contract", "test", "Run Mandos tests.")
     _add_project_arg(sub)
-    sub.add_argument("--directory", default="mandos", help="🗀 the directory containing the tests (default: %(default)s)")
+    sub.add_argument("--directory", default="mandos",
+                     help="🗀 the directory containing the tests (default: %(default)s)")
     sub.add_argument("--wildcard", required=False, help="wildcard to match only specific test files")
     sub.set_defaults(func=run_tests)
 
-    sub = cli_shared.add_command_subparser(subparsers, "contract", "deploy", "Deploy a Smart Contract.")
+    sub = cli_shared.add_command_subparser(subparsers, "contract", "report", "Print a detailed report of the smart contracts.")
+    _add_project_arg(sub)
+    sub.add_argument("--skip-build", action="store_true", default=False, help="skips the step of building of the wasm contracts")
+    sub.add_argument("--skip-twiggy", action="store_true", default=False, help="skips the steps of building the debug wasm files and running twiggy")
+    sub.add_argument("--output-format", type=str, default="text-markdown", choices=["github-markdown", "text-markdown", "json"], help="report output format (default: %(default)s)")
+    sub.add_argument("--output-file", type=Path, help="if specified, the output is written to a file, otherwise it's written to the standard output")
+    sub.add_argument("--compare", type=Path, nargs='+', metavar=("report-1.json", "report-2.json"), help="create a comparison from two or more reports")
+    sub.set_defaults(func=projects.do_report)
+
+    output_description = CLIOutputBuilder.describe(with_contract=True, with_transaction_on_network=True, with_simulation=True)
+    sub = cli_shared.add_command_subparser(subparsers, "contract", "deploy", f"Deploy a Smart Contract.{output_description}")
     _add_project_or_bytecode_arg(sub)
     _add_metadata_arg(sub)
     cli_shared.add_outfile_arg(sub)
@@ -62,7 +88,8 @@ def setup_parser(args: List[str], subparsers: Any) -> Any:
 
     sub.set_defaults(func=deploy)
 
-    sub = cli_shared.add_command_subparser(subparsers, "contract", "call", "Interact with a Smart Contract (execute function).")
+    sub = cli_shared.add_command_subparser(subparsers, "contract", "call",
+                                           f"Interact with a Smart Contract (execute function).{output_description}")
     _add_contract_arg(sub)
     cli_shared.add_outfile_arg(sub)
     cli_shared.add_wallet_args(args, sub)
@@ -78,7 +105,8 @@ def setup_parser(args: List[str], subparsers: Any) -> Any:
 
     sub.set_defaults(func=call)
 
-    sub = cli_shared.add_command_subparser(subparsers, "contract", "upgrade", "Upgrade a previously-deployed Smart Contract")
+    sub = cli_shared.add_command_subparser(subparsers, "contract", "upgrade",
+                                           f"Upgrade a previously-deployed Smart Contract.{output_description}")
     _add_contract_arg(sub)
     cli_shared.add_outfile_arg(sub)
     _add_project_or_bytecode_arg(sub)
@@ -95,7 +123,8 @@ def setup_parser(args: List[str], subparsers: Any) -> Any:
 
     sub.set_defaults(func=upgrade)
 
-    sub = cli_shared.add_command_subparser(subparsers, "contract", "query", "Query a Smart Contract (call a pure function)")
+    sub = cli_shared.add_command_subparser(subparsers, "contract", "query",
+                                           "Query a Smart Contract (call a pure function)")
     _add_contract_arg(sub)
     cli_shared.add_proxy_arg(sub)
     _add_function_arg(sub)
@@ -107,7 +136,12 @@ def setup_parser(args: List[str], subparsers: Any) -> Any:
 
 
 def _add_project_arg(sub: Any):
-    sub.add_argument("project", nargs='?', default=os.getcwd(), help="🗀 the project directory (default: current directory)")
+    sub.add_argument("project", nargs='?', default=os.getcwd(),
+                     help="🗀 the project directory (default: current directory)")
+
+
+def _add_recursive_arg(sub: Any):
+    sub.add_argument("-r", "--recursive", dest="recursive", action="store_true", help="locate projects recursively")
 
 
 def _add_project_or_bytecode_arg(sub: Any):
@@ -127,12 +161,20 @@ def _add_function_arg(sub: Any):
 
 
 def _add_arguments_arg(sub: Any):
-    sub.add_argument("--arguments", nargs='+', help="arguments for the contract transaction, as numbers or hex-encoded. E.g. --arguments 42 0x64 1000 0xabba")
+    sub.add_argument("--arguments", nargs='+',
+                     help="arguments for the contract transaction, as [number, bech32-address, ascii string, "
+                          "boolean] or hex-encoded. E.g. --arguments 42 0x64 1000 0xabba str:TOK-a1c2ef true erd1[..]")
 
 
 def _add_metadata_arg(sub: Any):
-    sub.add_argument("--metadata-not-upgradeable", dest="metadata_upgradeable", action="store_false", help="‼ mark the contract as NOT upgradeable (default: upgradeable)")
-    sub.add_argument("--metadata-payable", dest="metadata_payable", action="store_true", help="‼ mark the contract as payable (default: not payable)")
+    sub.add_argument("--metadata-not-upgradeable", dest="metadata_upgradeable", action="store_false",
+                     help="‼ mark the contract as NOT upgradeable (default: upgradeable)")
+    sub.add_argument("--metadata-not-readable", dest="metadata_readable", action="store_false",
+                     help="‼ mark the contract as NOT readable (default: readable)")
+    sub.add_argument("--metadata-payable", dest="metadata_payable", action="store_true",
+                     help="‼ mark the contract as payable (default: not payable)")
+    sub.add_argument("--metadata-payable-by-sc", dest="metadata_payable_by_sc", action="store_true",
+                     help="‼ mark the contract as payable by SC (default: not payable by SC)")
     sub.set_defaults(metadata_upgradeable=True, metadata_payable=False)
 
 
@@ -143,28 +185,41 @@ def list_templates(args: Any):
 def create(args: Any):
     name = args.name
     template = args.template
-    directory = args.directory
+    directory = Path(args.directory)
 
     projects.create_from_template(name, template, directory)
 
 
+def get_project_paths(args: Any) -> List[Path]:
+    base_path = Path(args.project)
+    recursive = bool(args.recursive)
+    if recursive:
+        return get_project_paths_recursively(base_path)
+    return [base_path]
+
+
 def clean(args: Any):
-    project = args.project
-    projects.clean_project(project)
+    project_paths = get_project_paths(args)
+    for project in project_paths:
+        projects.clean_project(project)
 
 
 def build(args: Any):
-    project = args.project
+    project_paths = get_project_paths(args)
     options = {
         "debug": args.debug,
         "optimized": not args.no_optimization,
+        "no-wasm-opt": args.no_wasm_opt,
         "verbose": args.verbose,
         "cargo_target_dir": args.cargo_target_dir,
         "wasm_symbols": args.wasm_symbols,
-        "wasm_name": args.wasm_name
+        "wasm_name": args.wasm_name,
+        "skip-eei-checks": args.skip_eei_checks,
+        "ignore-eei-checks": args.ignore_eei_checks
     }
 
-    projects.build_project(project, options)
+    for project in project_paths:
+        projects.build_project(project, options)
 
 
 def run_tests(args: Any):
@@ -189,52 +244,28 @@ def deploy(args: Any):
     logger.info("Contract address: %s", contract.address)
     utils.log_explorer_contract_address(chain, contract.address)
 
-    result = None
-    try:
-        result = _send_or_simulate(tx, args)
-    finally:
-        txdict = tx.to_dump_dict()
-        txdict['address'] = contract.address.bech32()
-        dump_tx_and_result(txdict, result, args)
-
-
-def dump_tx_and_result(tx: Any, result: Any, args: Any):
-    dump = dict()
-    dump['emitted_tx'] = tx
-
-    try:
-        returnCode = ''
-        returnMessage = ''
-        dump['result'] = result['result']
-        for scrHash, scr in dump['result']['scResults'].items():
-            if scr['receiver'] == tx['tx']['sender']:
-                retCode = scr['data'][1:]
-                retCode = bytes.fromhex(retCode).decode('ascii')
-                returnCode = retCode
-                returnMessage = scr['returnMessage']
-
-        dump['result']['returnCode'] = returnCode
-        dump['result']['returnMessage'] = returnMessage
-    except TypeError:
-        pass
-
-    utils.dump_out_json(dump, outfile=args.outfile)
+    _send_or_simulate(tx, contract, args)
 
 
 def _prepare_contract(args: Any) -> SmartContract:
     if args.bytecode and len(args.bytecode):
         bytecode = utils.read_binary_file(Path(args.bytecode)).hex()
     else:
-        project = load_project(args.project)
+        project_path = Path(args.project)
+        project = load_project(project_path)
         bytecode = project.get_bytecode()
 
-    metadata = CodeMetadata(args.metadata_upgradeable, args.metadata_payable)
+    metadata = CodeMetadata(upgradeable=args.metadata_upgradeable, readable=args.metadata_readable,
+                            payable=args.metadata_payable, payable_by_sc=args.metadata_payable_by_sc)
     contract = SmartContract(bytecode=bytecode, metadata=metadata)
     return contract
 
 
 def _prepare_sender(args: Any) -> Account:
-    if args.pem:
+    sender: Account
+    if args.ledger:
+        sender = LedgerAccount(account_index=args.ledger_account_index, address_index=args.ledger_address_index)
+    elif args.pem:
         sender = Account(pem_file=args.pem, pem_index=args.pem_index)
     elif args.keyfile and args.passfile:
         sender = Account(key_file=args.keyfile, pass_file=args.passfile)
@@ -265,11 +296,7 @@ def call(args: Any):
     sender = _prepare_sender(args)
 
     tx = contract.execute(sender, function, arguments, gas_price, gas_limit, value, chain, version)
-    try:
-        result = _send_or_simulate(tx, args)
-    finally:
-        txdict = tx.to_dump_dict()
-        dump_tx_and_result(txdict, result, args)
+    _send_or_simulate(tx, contract, args)
 
 
 def upgrade(args: Any):
@@ -289,11 +316,7 @@ def upgrade(args: Any):
     sender = _prepare_sender(args)
 
     tx = contract.upgrade(sender, arguments, gas_price, gas_limit, value, chain, version)
-    try:
-        result = _send_or_simulate(tx, args)
-    finally:
-        txdict = tx.to_dump_dict()
-        dump_tx_and_result(txdict, result, args)
+    _send_or_simulate(tx, contract, args)
 
 
 def query(args: Any):
@@ -308,18 +331,7 @@ def query(args: Any):
     utils.dump_out_json(result)
 
 
-def _send_or_simulate(tx: Transaction, args: Any):
-    send_wait_result = args.wait_result and args.send and not args.simulate
-    send_only = args.send and not (args.wait_result or args.simulate)
-    simulate = args.simulate and not (send_only or send_wait_result)
-
-    if send_wait_result:
-        proxy = ElrondProxy(args.proxy)
-        response = tx.send_wait_result(proxy, args.timeout)
-        return None
-    elif send_only:
-        tx.send(ElrondProxy(args.proxy))
-        return None
-    elif simulate:
-        response = tx.simulate(ElrondProxy(args.proxy))
-        return response
+def _send_or_simulate(tx: Transaction, contract: SmartContract, args: Any):
+    output_builder = cli_shared.send_or_simulate(tx, args, dump_output=False)
+    output_builder.set_contract_address(contract.address)
+    utils.dump_out_json(output_builder.build(), outfile=args.outfile)
