@@ -1,9 +1,9 @@
-from itertools import chain
-from logging import Logger
 import os.path
+import semver
 from typing import Any, Dict, List
+from pathlib import Path
 
-from erdpy import errors, utils
+from erdpy import errors, utils, workstation
 
 ROOT_FOLDER_NAME = "elrondsdk"
 LOCAL_CONFIG_PATH = os.path.join(os.getcwd(), "erdpy.json")
@@ -12,6 +12,7 @@ GLOBAL_CONFIG_PATH = os.path.expanduser("~/elrondsdk/erdpy.json")
 DEFAULT_GAS_PRICE = 1000000000
 GAS_PER_DATA_BYTE = 1500
 MIN_GAS_LIMIT = 50000
+MAX_GAS_LIMIT = 600000000
 
 
 class MetaChainSystemSCsCost:
@@ -57,7 +58,8 @@ def get_dependency_url(key: str, tag: str, platform: str) -> str:
 def get_value(name: str) -> str:
     _guard_valid_name(name)
     data = get_active()
-    value = data.get(name, get_defaults()[name])
+    default_value = get_defaults()[name]
+    value = data.get(name, default_value)
     assert isinstance(value, str)
     return value
 
@@ -72,12 +74,24 @@ def set_value(name: str, value: Any):
     write_file(data)
 
 
-def get_active():
+def delete_value(name: str):
+    _guard_valid_config_deletion(name)
+    data = read_file()
+    active_config = data.get("active", "default")
+    data.setdefault("configurations", {})
+    data["configurations"].setdefault(active_config, {})
+    del data["configurations"][active_config][name]
+    write_file(data)
+
+
+def get_active() -> Dict[str, Any]:
     data = read_file()
     configs = data.get("configurations", {})
-    active_config = data.get("active", "default")
+    active_config_name: str = data.get("active", "default")
+    empty_config: Dict[str, Any] = dict()
+    result: Dict[str, Any] = configs.get(active_config_name, empty_config)
 
-    return configs.get(active_config, {})
+    return result;
 
 
 def set_active(name: str):
@@ -137,30 +151,40 @@ def get_defaults() -> Dict[str, Any]:
         "proxy": "https://testnet-gateway.elrond.com",
         "chainID": "T",
         "txVersion": "1",
-        "dependencies.arwentools.tag": "v1.1.2",
-        "dependencies.elrond_wasm_rs.tag": "v0.12.0",
-        "dependencies.arwentools.urlTemplate.linux": "https://github.com/ElrondNetwork/arwen-wasm-vm/archive/{TAG}.tar.gz",
-        "dependencies.arwentools.urlTemplate.osx": "https://github.com/ElrondNetwork/arwen-wasm-vm/archive/{TAG}.tar.gz",
+        "dependencies.vmtools.tag": "latest",
+        "dependencies.elrond_wasm_rs.tag": "latest",
+        "dependencies.vmtools.urlTemplate.linux": "https://github.com/ElrondNetwork/wasm-vm/archive/{TAG}.tar.gz",
+        "dependencies.vmtools.urlTemplate.osx": "https://github.com/ElrondNetwork/wasm-vm/archive/{TAG}.tar.gz",
         "dependencies.llvm.tag": "v9-19feb",
         "dependencies.llvm.urlTemplate.linux": "https://ide.elrond.com/vendor-llvm/{TAG}/linux-amd64.tar.gz?t=19feb",
         "dependencies.llvm.urlTemplate.osx": "https://ide.elrond.com/vendor-llvm/{TAG}/darwin-amd64.tar.gz?t=19feb",
-        "dependencies.rust.tag": "",
+        "dependencies.rust.tag": "nightly",
         "dependencies.nodejs.tag": "v12.18.3",
         "dependencies.nodejs.urlTemplate.linux": "https://nodejs.org/dist/{TAG}/node-{TAG}-linux-x64.tar.gz",
         "dependencies.nodejs.urlTemplate.osx": "https://nodejs.org/dist/{TAG}/node-{TAG}-darwin-x64.tar.gz",
-        "dependencies.elrond_go.tag": "master",
+        "dependencies.elrond_go.tag": "latest",
         "dependencies.elrond_go.urlTemplate.linux": "https://github.com/ElrondNetwork/elrond-go/archive/{TAG}.tar.gz",
         "dependencies.elrond_go.urlTemplate.osx": "https://github.com/ElrondNetwork/elrond-go/archive/{TAG}.tar.gz",
         "dependencies.elrond_go.url": "https://github.com/ElrondNetwork/elrond-go/archive/{TAG}.tar.gz",
-        "dependencies.elrond_proxy_go.tag": "master",
+        "dependencies.elrond_proxy_go.tag": "latest",
         "dependencies.elrond_proxy_go.urlTemplate.linux": "https://github.com/ElrondNetwork/elrond-proxy-go/archive/{TAG}.tar.gz",
         "dependencies.elrond_proxy_go.urlTemplate.osx": "https://github.com/ElrondNetwork/elrond-proxy-go/archive/{TAG}.tar.gz",
         "dependencies.golang.tag": "go1.15.2",
         "dependencies.golang.urlTemplate.linux": "https://golang.org/dl/{TAG}.linux-amd64.tar.gz",
         "dependencies.golang.urlTemplate.osx": "https://golang.org/dl/{TAG}.darwin-amd64.tar.gz",
-        "dependencies.mcl_signer.tag": "v1.0.0",
+        "dependencies.mcl_signer.tag": "latest",
         "dependencies.mcl_signer.urlTemplate.linux": "https://github.com/ElrondNetwork/elrond-sdk-go-tools/releases/download/{TAG}/mcl_signer_{TAG}_ubuntu-latest.tar.gz",
         "dependencies.mcl_signer.urlTemplate.osx": "https://github.com/ElrondNetwork/elrond-sdk-go-tools/releases/download/{TAG}/mcl_signer_{TAG}_macos-latest.tar.gz",
+        "dependencies.wasm-opt.tag": "latest",
+        "dependencies.twiggy.tag": "latest",
+        "dependencies.wabt.tag": "1.0.27",
+        "dependencies.wabt.urlTemplate.linux": "https://github.com/WebAssembly/wabt/releases/download/{TAG}/wabt-{TAG}-ubuntu.tar.gz",
+        "dependencies.wabt.urlTemplate.osx": "https://github.com/WebAssembly/wabt/releases/download/{TAG}/wabt-{TAG}-macos.tar.gz",
+        "dependencies.testwallets.tag": "latest",
+        "dependencies.testwallets.urlTemplate.linux": "https://github.com/ElrondNetwork/elrond-sdk-testwallets/archive/{TAG}.tar.gz",
+        "dependencies.testwallets.urlTemplate.osx": "https://github.com/ElrondNetwork/elrond-sdk-testwallets/archive/{TAG}.tar.gz",
+        "testnet.validate_expected_keys": "false",
+        "github_api_token": "",
     }
 
 
@@ -173,7 +197,8 @@ def resolve_config_path() -> str:
 def read_file() -> Dict[str, Any]:
     config_path = resolve_config_path()
     if os.path.isfile(config_path):
-        return utils.read_json_file(config_path)
+        data: Dict[str, Any] = utils.read_json_file(config_path)
+        return data
     return dict()
 
 
@@ -227,3 +252,45 @@ def determine_final_args(argv: List[str], config_args: Dict[str, Any]) -> List[s
         pre_args = [verbose_flag]
 
     return pre_args + argv + extra_args
+
+
+def get_dependency_directory(key: str, tag: str) -> Path:
+    parent_directory = get_dependency_parent_directory(key)
+    if tag == 'latest':
+        tag = get_latest_semver_from_directory(parent_directory)
+
+    return parent_directory / tag
+
+
+def get_dependency_parent_directory(key: str) -> Path:
+    tools_folder = Path(workstation.get_tools_folder())
+    return tools_folder / key
+
+
+def get_latest_semver_from_directory(directory: Path) -> str:
+    subdirs = [subdir.name for subdir in directory.iterdir()]
+    try:
+        return get_latest_semver(subdirs)
+    except IndexError:
+        raise Exception(f'no versions found in {directory}')
+
+
+def get_latest_semver(versions: List[str]) -> str:
+    semantic_versions = parse_strings_to_semver(versions)
+    latest_version = sorted(semantic_versions).pop()
+    return 'v' + str(latest_version)
+
+
+def parse_strings_to_semver(version_strings: List[str]) -> List[semver.VersionInfo]:
+    versions = []
+    for version_string in version_strings:
+        try:
+            # Omit the 'v' prefix of the version string
+            version_string = version_string[1:]
+            version = semver.VersionInfo.parse(version_string)
+        except ValueError:
+            continue
+
+        versions.append(version)
+
+    return versions
