@@ -1,4 +1,5 @@
 
+from pathlib import Path
 from typing import Any, Union
 
 
@@ -26,6 +27,11 @@ class TemplateMissingError(KnownError):
         super().__init__(f"Template missing: {template}")
 
 
+class BadTemplateError(KnownError):
+    def __init__(self, directory: Path):
+        super().__init__(f"Bad template: {directory}")
+
+
 class DownloadError(KnownError):
     pass
 
@@ -63,6 +69,11 @@ class NotSupportedProject(KnownError):
         super().__init__(f"Directory is not a supported project: {directory}")
 
 
+class NotSupportedProjectFeature(KnownError):
+    def __init__(self):
+        super().__init__(f"Project feature not yet supported.")
+
+
 class PlatformNotSupported(KnownError):
     def __init__(self, action_or_item: str, platform: str):
         super().__init__(f"[{action_or_item}] is not supported on platform [{platform}].")
@@ -73,11 +84,6 @@ class BuildError(KnownError):
         super().__init__(f"Build error: {message}.", inner)
 
 
-class BadSink(ProgrammingError):
-    def __init__(self, name: str):
-        super().__init__(f"Bad sink:\n {name}.")
-
-
 class UnknownArgumentFormat(KnownError):
     def __init__(self, argument: Any):
         super().__init__(f"Cannot handle non-hex, non-number arguments yet: {argument}.")
@@ -130,6 +136,10 @@ class ConfigurationProtectedError(KnownError):
         super().__init__(f"This configuration name is protected: {name}.")
 
 
+class UnsupportedConfigurationValue(KnownError):
+    pass
+
+
 class UnknownDerivationFunction(KnownError):
     def __init__(self):
         super().__init__("Unknown key derivation function.")
@@ -140,6 +150,11 @@ class UnknownCipher(KnownError):
         super().__init__(f"Unknown cipher: {name}.")
 
 
+class GasLimitTooLarge(KnownError):
+    def __init__(self, current: int, limit: int):
+        super().__init__(f"The gas limit provided ({current}) exceeds the max gas limit of allowed for a transaction ({limit})")
+
+
 class InvalidKeystoreFilePassword(KnownError):
     def __init__(self):
         super().__init__("Provided keystore file password is invalid.")
