@@ -1,9 +1,10 @@
 import logging
-import os
 import subprocess
 from os import path
 from pathlib import Path
 
+from typing import List
+
 from erdpy import dependencies, errors, myprocess, utils
 from erdpy.projects.project_base import Project
 
@@ -12,7 +13,7 @@ logger = logging.getLogger('ProjectClang')
 
 class ProjectClang(Project):
 
-    def __init__(self, directory):
+    def __init__(self, directory: Path):
         super().__init__(directory)
         self.ensure_config_file()
 
@@ -83,7 +84,7 @@ class ProjectClang(Project):
             tool,
             '--no-entry',
             str(self.file_o),
-            '-o', self.file_output,
+            '-o', str(self.file_output),
             '--strip-all',
             '-allow-undefined'
         ]
@@ -98,7 +99,7 @@ class ProjectClang(Project):
 
         myprocess.run_process(args)
 
-    def _do_after_build(self) -> Path:
+    def _do_after_build_custom(self) -> List[Path]:
         output_wasm_file = self._copy_to_output(self.file_output)
         self.file_output.unlink()
         self.file_ll.unlink()
@@ -108,7 +109,7 @@ class ProjectClang(Project):
                 ll_file.unlink()
             except FileNotFoundError:
                 pass
-        return output_wasm_file
+        return [output_wasm_file]
 
     def _get_llvm_path(self):
         return dependencies.get_module_directory('llvm')
@@ -135,7 +136,7 @@ class ProjectClang(Project):
 
         self.config['source_files'] = source_files
 
-    def get_exported_functions(self):
+    def get_exported_functions(self) -> List[str]:
         file_export = self.find_file_globally('*.export')
         lines = utils.read_lines(file_export)
         return lines
