@@ -4,21 +4,16 @@ import logging
 from collections import OrderedDict
 from typing import Any, Dict, List, TextIO
 
-from erdpy import errors, utils
-from erdpy.accounts import Account, Address
-from erdpy.interfaces import IElrondProxy, ITransaction
-from erdpy.ledger.config import compare_versions
-from erdpy.ledger.ledger_app_handler import SIGN_USING_HASH_VERSION
-from erdpy.ledger.ledger_functions import do_get_ledger_version, TX_HASH_SIGN_VERSION, TX_HASH_SIGN_OPTIONS, \
-    do_sign_transaction_with_ledger
-from erdpy.wallet import signing
+from erdpy import config, errors, utils
+from erdpy.accounts import Account, Address, LedgerAccount
+from erdpy.interfaces import IElrondProxy, ITransaction, ITransactionOnNetwork
 
 logger = logging.getLogger("transactions")
 
 
 class Transaction(ITransaction):
     def __init__(self):
-        self.hash = ""
+        self.hash: str = ""
         self.nonce = 0
         self.value = "0"
         self.receiver = ""
@@ -27,7 +22,7 @@ class Transaction(ITransaction):
         self.receiverUsername = ""
         self.gasPrice = 0
         self.gasLimit = 0
-        self.data = ""
+        self.data: str = ""
         self.chainID = ""
         self.version = 0
         self.options = 0
@@ -54,15 +49,20 @@ class Transaction(ITransaction):
         return self._field_decoded("receiverUsername")
 
     def _field_encoded(self, field: str) -> str:
-        bytes = self.__dict__.get(field, None).encode("utf-8")
-        encoded = base64.b64encode(bytes).decode()
+        field_bytes = self.__dict__.get(field, None).encode("utf-8")
+        encoded = base64.b64encode(field_bytes).decode()
         return encoded
 
     def _field_decoded(self, field: str) -> str:
         return base64.b64decode(self.__dict__.get(field, None)).decode()
 
     def sign(self, account: Account):
-        self.signature = signing.sign_transaction(self, account)
+        self.validate()
+        self.signature = account.sign_transaction(self)
+
+    def validate(self) -> None:
+        if self.gasLimit > config.MAX_GAS_LIMIT:
+            raise errors.GasLimitTooLarge(self.gasLimit, config.MAX_GAS_LIMIT)
 
     def serialize(self) -> bytes:
         dictionary = self.to_dictionary()
@@ -90,22 +90,6 @@ class Transaction(ITransaction):
         instance.receiverUsername = instance.receiver_username_encoded()
         return instance
 
-    def to_dump_dict(self, extra: Any = {}):
-        dump_dict: Dict[str, Any] = dict()
-        dump_dict['tx'] = self.to_dictionary()
-        dump_dict['hash'] = self.hash or ""
-        dump_dict['data'] = self.data
-        dump_dict.update(extra)
-        return dump_dict
-
-    def dump_to(self, f: Any, extra: Any = {}):
-        dump: Any = utils.Object()
-        dump.tx = self.to_dictionary()
-        dump.hash = self.hash or ""
-        dump.data = self.data
-        dump.__dict__.update(extra)
-        f.writelines([dump.to_json(), "\n"])
-
     def send(self, proxy: IElrondProxy):
         if not self.signature:
             raise errors.TransactionIsNotSigned()
@@ -118,20 +102,13 @@ class Transaction(ITransaction):
         utils.log_explorer_transaction(self.chainID, self.hash)
         return self.hash
 
-    def send_wait_result(self, proxy, timeout):
+    def send_wait_result(self, proxy: IElrondProxy, timeout: int) -> ITransactionOnNetwork:
         if not self.signature:
             raise errors.TransactionIsNotSigned()
 
-        response = proxy.send_transaction_and_wait_for_result(self.to_dictionary(), timeout)
-        self.hash = response['hash']
-        return response
-
-    def simulate(self, proxy: IElrondProxy):
-        if not self.signature:
-            raise errors.TransactionIsNotSigned()
-
-        dictionary = self.to_dictionary()
-        return proxy.simulate_transaction(dictionary)
+        txOnNetwork = proxy.send_transaction_and_wait_for_result(self.to_dictionary(), timeout)
+        self.hash = txOnNetwork.get_hash()
+        return txOnNetwork
 
     def to_dictionary(self) -> Dict[str, Any]:
         dictionary: Dict[str, Any] = OrderedDict()
@@ -179,6 +156,18 @@ class Transaction(ITransaction):
     def wrap_inner(self, inner: ITransaction) -> None:
         self.data = inner.serialize_as_inner()
 
+    def set_version(self, version: int):
+        self.version = version
+
+    def set_options(self, options: int):
+        self.options = options
+
+    def get_data(self) -> str:
+        return self.data
+    
+    def get_hash(self) -> str:
+        return self.hash
+
 
 class BunchOfTransactions:
     def __init__(self):
@@ -219,7 +208,7 @@ class BunchOfTransactions:
 def do_prepare_transaction(args: Any) -> Transaction:
     account = Account()
     if args.ledger:
-        return do_prepare_transaction_ledger(args)
+        account = LedgerAccount(account_index=args.ledger_account_index, address_index=args.ledger_address_index)
     if args.pem:
         account = Account(pem_file=args.pem, pem_index=args.pem_index)
     elif args.keyfile and args.passfile:
@@ -241,43 +230,3 @@ def do_prepare_transaction(args: Any) -> Transaction:
 
     tx.sign(account)
     return tx
-
-
-def do_prepare_transaction_ledger(args: Any) -> Transaction:
-    account_index = 0
-    address_index = 0
-    if args.ledger_account_index:
-        account_index = args.ledger_account_index
-    if args.ledger_address_index:
-        address_index = args.ledger_address_index
-
-    tx = Transaction()
-    tx.nonce = int(args.nonce)
-    tx.value = args.value
-    tx.receiver = args.receiver
-    tx.sender = args.ledger_address
-    tx.senderUsername = getattr(args, "sender_username", None)
-    tx.receiverUsername = getattr(args, "receiver_username", None)
-    tx.gasPrice = int(args.gas_price)
-    tx.gasLimit = int(args.gas_limit)
-    tx.data = args.data
-    tx.chainID = args.chain
-
-    ledger_version = do_get_ledger_version()
-    should_use_hash_signing = compare_versions(ledger_version, SIGN_USING_HASH_VERSION) >= 0
-    if should_use_hash_signing:
-        tx.version = TX_HASH_SIGN_VERSION
-        tx.options = TX_HASH_SIGN_OPTIONS
-
-    tx.signature = ""
-
-    signature = do_sign_transaction_with_ledger(
-        tx_payload=tx.serialize(),
-        account_index=account_index,
-        address_index=address_index,
-        sign_using_hash=should_use_hash_signing,
-    )
-
-    tx.signature = signature
-
-    return tx
