@@ -1,37 +1,45 @@
 import json
 import logging
 import os
-import os.path
 import pathlib
 import shutil
 import stat
 import sys
+import requests_cache
 import tarfile
 import zipfile
 from pathlib import Path
-from typing import Any, AnyStr, List, Union, Optional, cast, IO, Dict
+from typing import Any, List, Union, Optional, cast, IO, Dict
 
 import toml
 
+import erdpy.config
 from erdpy import errors
 
 logger = logging.getLogger("utils")
 
+class ISerializable:
+    def to_dictionary(self) -> Dict[str, Any]:
+        return self.__dict__
 
-class Object:
+
+class Object(ISerializable):
     def __repr__(self):
         return str(self.__dict__)
 
+    def to_dictionary(self):
+        return dict(self.__dict__)
+
     def to_json(self):
         data_json = json.dumps(self.__dict__, indent=4)
         return data_json
 
 
-class ObjectEncoder(json.JSONEncoder):
-    def default(self, obj):
-        if isinstance(obj, Object):
-            return obj.__dict__
-        return json.JSONEncoder.default(self, obj)
+class BasicEncoder(json.JSONEncoder):
+    def default(self, o: Any):
+        if isinstance(o, ISerializable):
+            return o.to_dictionary()
+        return json.JSONEncoder.default(self, o)
 
 
 def omit_fields(data: Any, fields: List[str] = []):
@@ -42,23 +50,23 @@ def omit_fields(data: Any, fields: List[str] = []):
     raise errors.ProgrammingError("omit_fields: only dictionaries are supported.")
 
 
-def untar(archive_path: str, destination_folder: str) -> None:
+def untar(archive_path: Path, destination_folder: Path) -> None:
     logger.debug(f"untar [{archive_path}] to [{destination_folder}].")
 
     ensure_folder(destination_folder)
-    tar = tarfile.open(archive_path)
-    tar.extractall(path=destination_folder)
+    tar = tarfile.open(str(archive_path))
+    tar.extractall(path=str(destination_folder))
     tar.close()
 
     logger.debug("untar done.")
 
 
-def unzip(archive_path, destination_folder):
+def unzip(archive_path: Path, destination_folder: Path):
     logger.debug(f"unzip [{archive_path}] to [{destination_folder}].")
 
     ensure_folder(destination_folder)
-    with zipfile.ZipFile(archive_path, "r") as my_zip:
-        my_zip.extractall(destination_folder)
+    with zipfile.ZipFile(str(archive_path), "r") as my_zip:
+        my_zip.extractall(str(destination_folder))
 
     logger.debug("unzip done.")
 
@@ -67,7 +75,17 @@ def ensure_folder(folder: Union[str, Path]):
     pathlib.Path(folder).mkdir(parents=True, exist_ok=True)
 
 
-def read_lines(file: str):
+def uniquify(path: Path) -> Path:
+    '''Generates the next available non-already-existing filename, by adding a _1, _2, _3, etc. suffix before the extension if necessary'''
+    i = 1
+    stem = path.stem
+    while path.exists():
+        path = path.with_name(f"{stem}_{i}").with_suffix(path.suffix)
+        i += 1
+    return path
+
+
+def read_lines(file: Path) -> List[str]:
     with open(file) as f:
         lines = f.readlines()
     lines = [line.strip() for line in lines]
@@ -106,11 +124,9 @@ def read_text_file(path: Path) -> str:
         raise errors.BadFile(str(path), err) from None
 
 
-def write_file(f: Any, text: str):
-    if isinstance(f, str) or isinstance(f, pathlib.PosixPath):
-        with open(f, "w") as f:
-            return f.write(text)
-    return f.write(text)
+def write_file(file_path: Path, text: str):
+    with open(file_path, "w") as file:
+        return file.write(text)
 
 
 def read_toml_file(filename):
@@ -122,11 +138,9 @@ def write_toml_file(filename, data):
         toml.dump(data, f)
 
 
-def read_json_file(filename: Union[str, Path]) -> Dict[str, Any]:
-    data: Dict[str, Any]
+def read_json_file(filename: Union[str, Path]) -> Any:
     with open(filename) as f:
-        data = json.load(f)
-    return data
+        return json.load(f)
 
 
 def write_json_file(filename: str, data: Any):
@@ -138,7 +152,7 @@ def dump_out_json(data: Any, outfile: Any = None):
     if not outfile:
         outfile = sys.stdout
 
-    json.dump(data, outfile, indent=4, cls=ObjectEncoder)
+    json.dump(data, outfile, indent=4, cls=BasicEncoder)
     outfile.write("\n")
 
 
@@ -147,7 +161,7 @@ def prettify_json_file(filename: str):
     write_json_file(filename, data)
 
 
-def get_subfolders(folder):
+def get_subfolders(folder: Path) -> List[str]:
     return [item.name for item in os.scandir(folder) if item.is_dir() and not item.name.startswith(".")]
 
 
@@ -168,12 +182,13 @@ def find_in_dictionary(dictionary, compound_path):
     return node
 
 
-def list_files(folder: Union[str, Path], suffix: Optional[str] = None) -> List[str]:
-    files = os.listdir(folder)
-    files = [os.path.join(folder, f) for f in files]
+def list_files(folder: Path, suffix: Optional[str] = None) -> List[Path]:
+    folder = folder.expanduser()
+    files: List[Path] = [folder / file for file in os.listdir(folder)]
+    files = [file for file in files if file.is_file()]
 
     if suffix:
-        files = [e for e in files if e.lower().endswith(suffix.lower())]
+        files = [file for file in files if str(file).lower().endswith(suffix.lower())]
 
     return files
 
@@ -183,7 +198,7 @@ def remove_folder(folder: Union[str, Path]):
 
 
 def symlink(real: str, link: str) -> None:
-    if os.path.exists(link):
+    if os.path.islink(link):
         os.remove(link)
     os.symlink(real, link)
 
@@ -225,6 +240,30 @@ def parse_keys(bls_public_keys):
     return parsed_keys, len(keys)
 
 
+def query_latest_release_tag(repo: str) -> str:
+    """
+    Queries the Github API to retrieve the latest released tag of the specified
+    repository. The repository must be of the form 'organisation/project'.
+    """
+    url = f'https://api.github.com/repos/{repo}/releases'
+
+    github_api_token = erdpy.config.get_value('github_api_token')
+    headers = dict()
+    if github_api_token != '':
+        headers['Authorization'] = f'token {github_api_token}'
+
+    session = requests_cache.CachedSession('erdpy_requests_cache', use_cache_dir=True, cache_control=True)
+    response = session.get(url, headers=headers)
+    response.raise_for_status()
+
+    release_tags = [str(release['tag_name']) for release in response.json()]
+    try:
+        latest_release_tag = erdpy.config.get_latest_semver(release_tags)
+        return latest_release_tag
+    except IndexError:
+        raise Exception(f"no releases in {repo}")
+
+
 # https://code.visualstudio.com/docs/python/debugging
 def breakpoint():
     import debugpy
