@@ -1,7 +1,9 @@
-import binascii
 import logging
 from os import path
-from typing import Any
+from pathlib import Path
+from typing import Any, List, Tuple, Union
+from erdpy.contracts import SmartContract
+from erdpy.errors import BadUsage
 
 from erdpy.validators.validators_file import ValidatorsFile
 from erdpy import utils
@@ -20,36 +22,49 @@ def prepare_args_for_stake(args: Any):
         prepare_args_for_top_up(args)
         return
 
-    validators_file = ValidatorsFile(args.validators_file)
-
-    reward_address = args.reward_address
-
-    # TODO: Refactor, so that only address is received here.
     if args.pem:
-        account = Account(pem_file=args.pem)
+        node_operator = Account(pem_file=args.pem)
     elif args.keyfile and args.passfile:
-        account = Account(key_file=args.keyfile, pass_file=args.passfile)
+        node_operator = Account(key_file=args.keyfile, pass_file=args.passfile)
+    else:
+        raise BadUsage("cannot initialize node operator")
 
-    num_of_nodes = validators_file.get_num_of_nodes()
-    stake_data = 'stake@' + binascii.hexlify(num_of_nodes.to_bytes(1, byteorder="little")).decode()
-    validators_list = validators_file.get_validators_list()
-    for validator in validators_list:
-        # get validator
-        validator_pem = validator.get("pemFile")
-        validator_pem = path.join(path.dirname(args.validators_file), validator_pem)
-        seed, bls_key = parse_validator_pem(validator_pem)
-        signed_message = sign_message_with_bls_key(account.address.pubkey().hex(), seed.decode('ascii'))
-        stake_data += f"@{bls_key}@{signed_message}"
-
-    if reward_address:
-        reward_address = Address(args.reward_address)
-        stake_data += '@' + reward_address.hex()
+    validators_file_path = Path(args.validators_file)
+    reward_address = Address(args.reward_address) if args.reward_address else None
 
+    data, gas_limit = prepare_transaction_data_for_stake(node_operator.address, validators_file_path, reward_address)
+    args.data = data
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
-    args.data = stake_data
 
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.STAKE, num_of_nodes)
+        args.gas_limit = gas_limit
+
+
+def prepare_transaction_data_for_stake(node_operator_address: Address, validators_file_path: Path, reward_address: Union[Address, None]) -> Tuple[str, int]:
+    validators_file = ValidatorsFile(validators_file_path)
+    num_of_nodes = validators_file.get_num_of_nodes()
+    validators_list = validators_file.get_validators_list()
+
+    call_arguments: List[Any] = []
+    call_arguments.append(num_of_nodes)
+
+    for validator in validators_list:
+        # Get path of "pemFile", make it absolute
+        validator_pem = Path(validator.get("pemFile")).expanduser()
+        validator_pem = validator_pem if validator_pem.is_absolute() else validators_file_path / validator_pem
+        
+        secret_key_bytes, bls_key = parse_validator_pem(Path(validator_pem))
+        signed_message = sign_message_with_bls_key(node_operator_address.hex(), secret_key_bytes.decode('ascii'))
+        call_arguments.append(f"0x{bls_key}")
+        call_arguments.append(f"0x{signed_message}")
+
+    if reward_address:
+        call_arguments.append(f"0x{reward_address.hex()}")
+
+    data = SmartContract().prepare_execute_transaction_data("stake", call_arguments)
+    gas_limit = estimate_system_sc_call(data, MetaChainSystemSCsCost.STAKE, num_of_nodes)
+
+    return data, gas_limit
 
 
 def prepare_args_for_top_up(args: Any):
@@ -57,7 +72,7 @@ def prepare_args_for_top_up(args: Any):
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
 
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.STAKE, 1)
+        args.gas_limit = estimate_system_sc_call(args.data, MetaChainSystemSCsCost.STAKE, 1)
 
 
 def prepare_args_for_unstake(args: Any):
@@ -66,7 +81,7 @@ def prepare_args_for_unstake(args: Any):
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
 
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.UNSTAKE, num_keys)
+        args.gas_limit = estimate_system_sc_call(args.data, MetaChainSystemSCsCost.UNSTAKE, num_keys)
 
 
 def prepare_args_for_unbond(args: Any):
@@ -75,7 +90,7 @@ def prepare_args_for_unbond(args: Any):
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
 
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.UNBOND, num_keys)
+        args.gas_limit = estimate_system_sc_call(args.data, MetaChainSystemSCsCost.UNBOND, num_keys)
 
 
 def prepare_args_for_unjail(args: Any):
@@ -84,7 +99,7 @@ def prepare_args_for_unjail(args: Any):
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
 
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.UNJAIL, num_keys)
+        args.gas_limit = estimate_system_sc_call(args.data, MetaChainSystemSCsCost.UNJAIL, num_keys)
 
 
 def prepare_args_for_change_reward_address(args: Any):
@@ -93,7 +108,7 @@ def prepare_args_for_change_reward_address(args: Any):
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
 
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.CHANGE_REWARD_ADDRESS)
+        args.gas_limit = estimate_system_sc_call(args.data, MetaChainSystemSCsCost.CHANGE_REWARD_ADDRESS)
 
 
 def prepare_args_for_claim(args: Any):
@@ -101,7 +116,7 @@ def prepare_args_for_claim(args: Any):
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
 
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.CLAIM)
+        args.gas_limit = estimate_system_sc_call(args.data, MetaChainSystemSCsCost.CLAIM)
 
 
 def prepare_args_for_unstake_nodes(args: Any):
@@ -110,7 +125,7 @@ def prepare_args_for_unstake_nodes(args: Any):
 
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.UNSTAKE, num_keys)
+        args.gas_limit = estimate_system_sc_call(args.data, MetaChainSystemSCsCost.UNSTAKE, num_keys)
 
 
 def prepare_args_for_unstake_tokens(args: Any):
@@ -119,7 +134,7 @@ def prepare_args_for_unstake_tokens(args: Any):
 
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.UNSTAKE_TOKENS)
+        args.gas_limit = estimate_system_sc_call(args.data, MetaChainSystemSCsCost.UNSTAKE_TOKENS)
 
 
 def prepare_args_for_unbond_nodes(args: Any):
@@ -128,7 +143,7 @@ def prepare_args_for_unbond_nodes(args: Any):
 
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.UNBOND, num_keys)
+        args.gas_limit = estimate_system_sc_call(args.data, MetaChainSystemSCsCost.UNBOND, num_keys)
 
 
 def prepare_args_for_unbond_tokens(args: Any):
@@ -137,7 +152,7 @@ def prepare_args_for_unbond_tokens(args: Any):
 
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.UNBOND_TOKENS)
+        args.gas_limit = estimate_system_sc_call(args.data, MetaChainSystemSCsCost.UNBOND_TOKENS)
 
 
 def prepare_args_for_clean_registered_data(args: Any):
@@ -145,7 +160,7 @@ def prepare_args_for_clean_registered_data(args: Any):
 
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.STAKE)
+        args.gas_limit = estimate_system_sc_call(args.data, MetaChainSystemSCsCost.STAKE)
 
 
 def prepare_args_for_restake_unstaked_nodes(args: Any):
@@ -154,11 +169,11 @@ def prepare_args_for_restake_unstaked_nodes(args: Any):
 
     args.receiver = VALIDATORS_SMART_CONTRACT_ADDRESS
     if args.estimate_gas:
-        args.gas_limit = estimate_system_sc_call(args, MetaChainSystemSCsCost.STAKE, num_keys)
+        args.gas_limit = estimate_system_sc_call(args.data, MetaChainSystemSCsCost.STAKE, num_keys)
 
 
-def estimate_system_sc_call(args, base_cost, factor=1):
-    num_bytes = len(args.data)
+def estimate_system_sc_call(transaction_data: str, base_cost: int, factor: int = 1):
+    num_bytes = len(transaction_data)
     gas_limit = MIN_GAS_LIMIT + num_bytes * GAS_PER_DATA_BYTE
     gas_limit += factor * base_cost
     return gas_limit
